#define STRICT
#define WINVER       0x0400 /* Not 0x0501, for VC6 compatibility */
#define _WIN32_WINNT 0x0400 /* Not 0x0501, for VC6 compatibility */
#include <Windows.h>

#include "OpenGLFrame.version"
